'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _rules = require('../constants/rules');

var _rules2 = _interopRequireDefault(_rules);

var _includes = require('lodash/includes');

var _includes2 = _interopRequireDefault(_includes);

var _isReactComponent = require('../utils/is-react-component');

var _isReactComponent2 = _interopRequireDefault(_isReactComponent);

var _typeOf = require('type-of');

var _typeOf2 = _interopRequireDefault(_typeOf);

var _memoize = require('../utils/memoize');

var _memoize2 = _interopRequireDefault(_memoize);

var _immutable = require('immutable');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Default properties.
 *
 * @type {Object}
 */

var DEFAULTS = {
  rules: []
};

/**
 * Schema.
 *
 * @type {Record}
 */

var Schema = function (_ref) {
  _inherits(Schema, _ref);

  function Schema() {
    _classCallCheck(this, Schema);

    return _possibleConstructorReturn(this, Object.getPrototypeOf(Schema).apply(this, arguments));
  }

  _createClass(Schema, [{
    key: 'transform',


    /**
     * Transform a `state` to abide by the schema.
     *
     * @param {State} state
     * @return {State}
     */

    value: function transform(state) {
      var _this2 = this;

      var document = state.document;

      var transform = state.transform();

      document.filterDescendantsDeep(function (node) {
        var rule = _this2.validateNode(node);
        if (rule) transform = rule.transform(transform, node, state);
      });

      var rule = this.validateNode(document);
      if (rule) transform = rule.transform(transform, document, state);

      var next = transform.apply({ snapshot: false });
      return next;
    }

    /**
     * Validate a `state` against the schema.
     *
     * @param {State} state
     * @return {State}
     */

  }, {
    key: 'validate',
    value: function validate(state) {
      var _this3 = this;

      return !!state.document.findDescendant(function (node) {
        return _this3.validateNode(node);
      });
    }

    /**
     * Validate a `node` against the schema, returning the rule that was not met
     * if the node is invalid, or null if the rule was valid.
     *
     * @param {Node} node
     * @return {Object || Void}
     */

  }, {
    key: 'validateNode',
    value: function validateNode(node) {
      return this.rules.filter(function (rule) {
        return rule.match(node);
      }).find(function (rule) {
        return !rule.validate(node);
      });
    }
  }, {
    key: 'kind',


    /**
     * Get the kind.
     *
     * @return {String} kind
     */

    get: function get() {
      return 'schema';
    }
  }], [{
    key: 'create',


    /**
     * Create a new `Schema` with `properties`.
     *
     * @param {Object} properties
     * @return {Schema} mark
     */

    value: function create() {
      var properties = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

      if (properties instanceof Schema) return properties;
      return new Schema(normalizeProperties(properties));
    }
  }]);

  return Schema;
}(new _immutable.Record(DEFAULTS));

/**
 * Memoize read methods.
 */

(0, _memoize2.default)(Schema.prototype, ['transform', 'validate', 'validateNode']);

/**
 * Normalize the `properties` of a schema.
 *
 * @param {Object} properties
 * @return {Object}
 */

function normalizeProperties(properties) {
  var rules = properties.rules;
  var nodes = properties.nodes;
  var marks = properties.marks;

  if (!rules) rules = [];

  // If there's a `nodes` property, it's the node rule shorthand dictionary.
  if (nodes) {
    for (var key in nodes) {
      var value = nodes[key];
      var rule = void 0;

      if ((0, _isReactComponent2.default)(value)) {
        rule.match = { type: key };
        rule.component = value;
      } else {
        rule = _extends({
          kinds: ['block', 'inline'],
          type: key
        }, value);
      }

      rules.push(rule);
    }
  }

  // If there's a `marks` property, it's the mark rule shorthand dictionary.
  if (marks) {
    for (var _key in marks) {
      var _value = marks[_key];
      var _rule = void 0;

      if (_rule.match) {
        _rule = _extends({
          kind: 'mark',
          type: _key
        }, _value);
      } else {
        _rule.match = { type: _key };
        _rule.component = _value;
      }

      rules.push(_rule);
    }
  }

  return {
    rules: rules.concat(_rules2.default).map(normalizeRule)
  };
}

/**
 * Normalize a `rule` object.
 *
 * @param {Object} rule
 * @return {Object}
 */

function normalizeRule(rule) {
  return {
    match: normalizeMatch(rule.match),
    validate: normalizeValidate(rule.validate),
    transform: normalizeTransform(rule.transform)
  };
}

/**
 * Normalize a `match` spec.
 *
 * @param {Function || Object || String} match
 * @return {Function}
 */

function normalizeMatch(match) {
  switch ((0, _typeOf2.default)(match)) {
    case 'function':
      return match;
    case 'boolean':
      return function () {
        return match;
      };
    case 'object':
      return normalizeSpec(match);
    default:
      {
        throw new Error('Invalid `match` spec: "' + match + '".');
      }
  }
}

/**
 * Normalize a `validate` spec.
 *
 * @param {Function || Object || String} validate
 * @return {Function}
 */

function normalizeValidate(validate) {
  switch ((0, _typeOf2.default)(validate)) {
    case 'function':
      return validate;
    case 'boolean':
      return function () {
        return validate;
      };
    case 'object':
      return normalizeSpec(validate);
    default:
      {
        throw new Error('Invalid `validate` spec: "' + validate + '".');
      }
  }
}

/**
 * Normalize a `transform` spec.
 *
 * @param {Function || Object || String} transform
 * @return {Function}
 */

function normalizeTransform(transform) {
  switch ((0, _typeOf2.default)(transform)) {
    case 'function':
      return transform;
    default:
      {
        throw new Error('Invalid `transform` spec: "' + transform + '".');
      }
  }
}

/**
 * Normalize a `spec` object.
 *
 * @param {Object} obj
 * @return {Boolean}
 */

function normalizeSpec(obj) {
  var spec = _extends({}, obj);
  var nodes = spec.nodes ? _extends({}, spec.nodes) : {};

  // Normalize the text argument.
  if (spec.text) {
    switch ((0, _typeOf2.default)(spec.text)) {
      case 'function':
        break;
      case 'string':
        spec.text = function (text) {
          return text == spec.text;
        };
        break;
      case 'regexp':
        spec.text = function (text) {
          return text.match(spec);
        };
        break;
      default:
        throw new Error('Invalid `text` spec argument: "' + spec.text + '".');
    }
  }

  // Normalize recursively for the node specs.
  if (nodes.exactlyOf) nodes.exactlyOf = nodes.exactlyOf.map(normalizeSpec);
  if (nodes.anyOf) nodes.anyOf = nodes.anyOf.map(normalizeSpec);
  if (nodes.noneOf) nodes.noneOf = nodes.noneOf.map(normalizeSpec);

  // Return a checking function.
  return function (node) {
    // Run the simple equality checks first.
    if (spec.kind != null && spec.kind != node.kind || spec.type != null && spec.type != node.type || spec.isVoid != null && spec.isVoid != node.type || nodes.length != null && node.nodes.size != nodes.length || nodes.minLength != null && node.nodes.size < nodes.minLength || nodes.maxLength != null && node.nodes.size > nodes.maxLength || spec.kinds != null && !(0, _includes2.default)(spec.kinds, node.kind) || spec.types != null && !(0, _includes2.default)(spec.types, node.type) || spec.length != null && node.length != spec.length || spec.minLength != null && node.length < spec.minLength || spec.maxLength != null && node.length > spec.maxLength) {
      return false;
    }

    // Run the text check next.
    if (spec.text != null && !spec.text(node.text)) {
      return false;
    }

    // Run the node recursive checks next, start with `exactlyOf`, which likely
    // has the  greatest chance of not matching.
    if (nodes.exactlyOf) {
      if (node.nodes.size != nodes.exactlyOf.length) return false;

      var matches = node.nodes.reduce(function (valid, child, i) {
        if (!valid) return false;
        var checker = nodes.exactlyOf[i];
        if (!checker) return false;
        return checker(child);
      }, true);

      if (!matches) return false;
    }

    // Run the `anyOf` next check.
    if (nodes.anyOf) {
      var _matches = node.nodes.reduce(function (valid, child) {
        if (!valid) return false;
        return nodes.anyOf.reduce(function (pass, checker) {
          if (pass) return true;
          return checker(child);
        }, false);
      }, true);

      if (!_matches) return false;
    }

    // Run the `noneOf` next check.
    if (nodes.noneOf) {
      var _matches2 = node.nodes.reduce(function (valid, child) {
        if (!valid) return false;
        return nodes.noneOf.reduce(function (pass, checker) {
          if (!pass) return false;
          return !checker(child);
        }, true);
      });

      if (!_matches2) return false;
    }

    return true;
  };
}

/**
 * Export.
 *
 * @type {Record}
 */

exports.default = Schema;